@@ -12,7 +12,7 @@ from sdssdb.peewee.sdss5db.catalogdb import (BestBrightest, Catalog,
                                              CatalogToAllWise, CatalogToTIC_v8,
                                              SkyMapperGaia, TIC_v8)
 
-from . import BaseCarton
+from target_selection.cartons import BaseCarton
 
 
 class MWM_Halo_Best_Brightest_Carton(BaseCarton):
@@ -46,7 +46,8 @@ class MWM_Halo_Best_Brightest_Carton(BaseCarton):
     name = 'mwm_halo_bb'
     mapper = 'MWM'
     category = 'science'
-    program = 'mwm_halo'
+    program = 'mwm_filler'
+    instrument = None
     cadence = None
     priority = None
 
@@ -93,7 +94,8 @@ class MWM_Halo_SkyMapper_Carton(BaseCarton):
     name = 'mwm_halo_sm'
     mapper = 'MWM'
     category = 'science'
-    program = 'mwm_halo'
+    program = 'mwm_filler'
+    instrument = None
     cadence = None
     priority = 6050
 
