@@ -6,6 +6,8 @@
 # @Filename: mwm_rv.py
 # @License: BSD 3-clause (http://www.opensource.org/licenses/BSD-3-Clause)
 
+import math
+
 import peewee
 from astropy.coordinates import Angle
 
@@ -155,7 +157,8 @@ class MWM_RV_Long_RM_Carton(BaseCarton):
     """
     name = 'mwm_rv_long_rm'
     category = 'science'
-    cadence = None
+    instrument = 'APOGEE'
+    cadence = None  # plate only
     program = 'mwm_rv'
     mapper = 'MWM'
     priority = 2571
@@ -187,7 +190,16 @@ class MWM_RV_Long_RM_Carton(BaseCarton):
         # all been put in a common epoch 2015.5.
 
         query = (Catalog
-                 .select(CatalogToTIC_v8.catalogid)
+                 .select(CatalogToTIC_v8.catalogid,
+                         SDSS_APOGEE_AllStarMerge_r13.apogee_id,
+                         SDSS_APOGEE_AllStarMerge_r13.nvisits,
+                         SDSS_APOGEE_AllStarMerge_r13.ra.alias('allstarmerge_ra'),
+                         SDSS_APOGEE_AllStarMerge_r13.dec.alias('allstarmerge_dec'),
+                         SDSS_APOGEE_AllStarMerge_r13.pmra.alias('allstarmerge_pmra'),
+                         SDSS_APOGEE_AllStarMerge_r13.pmdec.alias('allstarmerge_pmdec'),
+                         SDSS_APOGEE_AllStarMerge_r13.h,
+                         SDSS_APOGEE_AllStarMerge_r13.baseline,
+                         SDSS_APOGEE_AllStarMerge_r13.fields)
                  .join(CatalogToTIC_v8,
                        on=(Catalog.catalogid == CatalogToTIC_v8.catalogid))
                  .join(TIC_v8,
@@ -282,7 +294,8 @@ class MWM_RV_Long_Bplates_Carton(BaseCarton):
     """
     name = 'mwm_rv_long_bplates'
     category = 'science'
-    cadence = None
+    instrument = 'APOGEE'
+    cadence = None  # plate only
     program = 'mwm_rv'
     mapper = 'MWM'
     priority = 2526
@@ -291,10 +304,10 @@ class MWM_RV_Long_Bplates_Carton(BaseCarton):
     # SDSS_APOGEE_AllStarMerge_r13(CatalogdbModel)--->'sdss_apogeeallstarmerge_r13'
 
     def build_query(self, version_id, query_region=None):
-        # ra_hours_to_degrees = (360.0 / 24.0)
+        ra_hours_to_degrees = (360.0 / 24.0)
 
-        ra = [0] * 35
-        dec = [0] * 35
+        ra = [0] * 40
+        dec = [0] * 40
         ra[1] = 11.83254;  dec[1] = 85.251     # noqa: E702, E241
         ra[2] = 47.39417;  dec[2] = 39.50294   # noqa: E702, E241
         ra[3] = 72.4084;   dec[3] = 63.603     # noqa: E702, E241
@@ -316,47 +329,62 @@ class MWM_RV_Long_Bplates_Carton(BaseCarton):
         ra[19] = 240.575;  dec[19] = 28.09     # noqa: E702, E241
         ra[20] = 248.3458; dec[20] = -0.5336   # noqa: E702, E241
 
-        # 172+58_btx
-        ra[21] = 159.0315; dec[21] = 43.7589  # noqa: E702, E241
+        # M67_btx
+        ra[21] = 8.8695 * ra_hours_to_degrees; dec[21] = 11.8252  # noqa: E702, E241
 
-        # 175+65_btx
-        ra[22] = 167.712; dec[22] = 39.8448  # noqa: E702, E241
+        # M35N2158_btx
+        ra[22] = 6.126 * ra_hours_to_degrees; dec[22] = 24.32512  # noqa: E702, E241
 
-        # 232+66_btx
-        ra[23] = 170.031; dec[23] = 17.4326  # noqa: E702, E241
+        # NGC2632_btx
+        ra[23] = 8.6685 * ra_hours_to_degrees; dec[23] = 19.6596  # noqa: E702, E241
 
-        # 209+73_btx
-        ra[24] = 174.207; dec[24] = 26.9809  # noqa: E702, E241
+        # M3-RV
+        ra[24] = 13.7531 * ra_hours_to_degrees; dec[24] = 27.6256  # noqa: E702, E241
 
-        # 248+68_btx
-        ra[25] = 174.642; dec[25] = 13.4369  # noqa: E702, E241
+        # BOOTES1
+        ra[25] = 13.9907 * ra_hours_to_degrees; dec[25] = 14.7  # noqa: E702, E241
 
-        # 279+61_btx
-        ra[26] = 181.8465; dec[26] = 0.2  # noqa: E702, E241
+        # URMINOR
+        ra[26] = 15.1933 * ra_hours_to_degrees; dec[26] = 67.11  # noqa: E702, E241
 
-        # 246+77_btx
-        ra[27] = 181.9035; dec[27] = 19.9253  # noqa: E702, E241
+        # K2EB_C06_320+48_btx
+        ra[27] = 13.62 * ra_hours_to_degrees; dec[27] = -13.28  # noqa: E702, E241
 
-        # 290+76_btx
-        ra[28] = 189.867; dec[28] = 14.0728  # noqa: E702, E241
+        # GD1-1
+        ra[28] = 8.4333 * ra_hours_to_degrees; dec[28] = 0.1666  # noqa: E702, E241
 
-        # 296+61_btx
-        ra[29] = 189.921; dec[29] = -1.3413  # noqa: E702, E241
+        # GD1-2
+        ra[29] = 8.7667 * ra_hours_to_degrees; dec[29] = 11.8833  # noqa: E702, E241
 
-        # 120+67_btx
-        ra[30] = 194.2575; dec[30] = 49.9041  # noqa: E702, E241
+        # GD1-3
+        ra[30] = 9.25 * ra_hours_to_degrees; dec[30] = 22.3166  # noqa: E702, E241
 
-        # 104+77_btx
-        ra[31] = 197.9385; dec[31] = 38.743  # noqa: E702, E241
+        # GD1-4
+        ra[31] = 10.9333 * ra_hours_to_degrees; dec[31] = 48.7333  # noqa: E702, E241
 
-        # 041+78_btx
-        ra[32] = 205.5945; dec[32] = 28.1849  # noqa: E702, E241
+        # GD1-5
+        ra[32] = 14.4833 * ra_hours_to_degrees; dec[32] = 57.5166  # noqa: E702, E241
 
-        # 071+62_btx
-        ra[33] = 221.169; dec[33] = 41.0707  # noqa: E702, E241
+        # ORPHAN-5
+        ra[33] = 9.4833 * ra_hours_to_degrees; dec[33] = 47.8666  # noqa: E702, E241
 
-        # 063+55_btx
-        ra[34] = 231.8415; dec[34] = 39.0758  # noqa: E702, E241
+        # ORPHAN-4
+        ra[34] = 9.8167 * ra_hours_to_degrees; dec[34] = 38.0999  # noqa: E702, E241
+
+        # ORPHAN-3
+        ra[35] = 10.1167 * ra_hours_to_degrees; dec[35] = 26.0833  # noqa: E702, E241
+
+        # ORPHAN-2
+        ra[36] = 10.4167 * ra_hours_to_degrees; dec[36] = 15.0499  # noqa: E702, E241
+
+        # ORPHAN-1
+        ra[37] = 10.8833 * ra_hours_to_degrees; dec[37] = -3.0667  # noqa: E702, E241
+
+        # PAL5-2
+        ra[38] = 16.0333 * ra_hours_to_degrees; dec[38] = 6.1666  # noqa: E702, E241
+
+        # PAL5-1
+        ra[39] = 14.8333 * ra_hours_to_degrees; dec[39] = -7.1  # noqa: E702, E241
 
         ra_dec_condition = (
             peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[1], dec[1], 3) |
@@ -392,12 +420,26 @@ class MWM_RV_Long_Bplates_Carton(BaseCarton):
             peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[31], dec[31], 3) |
             peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[32], dec[32], 3) |
             peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[33], dec[33], 3) |
-            peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[34], dec[34], 3))
+            peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[34], dec[34], 3) |
+            peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[35], dec[35], 3) |
+            peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[36], dec[36], 3) |
+            peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[37], dec[37], 3) |
+            peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[38], dec[38], 3) |
+            peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[39], dec[39], 3))
 
 # We use *mwm_rv_long_condition to unpack the tuple mwm_rv_long_condition.
 # However, ra_dec_condition is not a tuple so it does not have a * in the front.
         query = (Catalog
-                 .select(CatalogToTIC_v8.catalogid)
+                 .select(CatalogToTIC_v8.catalogid,
+                         SDSS_APOGEE_AllStarMerge_r13.apogee_id,
+                         SDSS_APOGEE_AllStarMerge_r13.nvisits,
+                         SDSS_APOGEE_AllStarMerge_r13.ra.alias('allstarmerge_ra'),
+                         SDSS_APOGEE_AllStarMerge_r13.dec.alias('allstarmerge_dec'),
+                         SDSS_APOGEE_AllStarMerge_r13.pmra.alias('allstarmerge_pmra'),
+                         SDSS_APOGEE_AllStarMerge_r13.pmdec.alias('allstarmerge_pmdec'),
+                         SDSS_APOGEE_AllStarMerge_r13.h,
+                         SDSS_APOGEE_AllStarMerge_r13.baseline,
+                         SDSS_APOGEE_AllStarMerge_r13.fields)
                  .join(CatalogToTIC_v8,
                        on=(Catalog.catalogid == CatalogToTIC_v8.catalogid))
                  .join(TIC_v8,
@@ -412,7 +454,6 @@ class MWM_RV_Long_Bplates_Carton(BaseCarton):
                         CatalogToTIC_v8.best >> True,
                         *mwm_rv_long_condition,
                         ra_dec_condition,
-                        SDSS_APOGEE_AllStarMerge_r13.h < 12.2,
                         SDSS_APOGEE_AllStarMerge_r13.nvisits >= 6))
         # Below ra, dec and radius are in degrees
         # query_region[0] is ra of center of the region
@@ -463,7 +504,8 @@ class MWM_RV_Long_FPS_Carton(BaseCarton):
     """
     name = 'mwm_rv_long_fps'
     category = 'science'
-    cadence = None
+    instrument = 'APOGEE'
+    cadence = None  # cadence is set in post_process()
     program = 'mwm_rv'
     mapper = 'MWM'
     priority = 2500
@@ -474,7 +516,16 @@ class MWM_RV_Long_FPS_Carton(BaseCarton):
     def build_query(self, version_id, query_region=None):
 
         query = (Catalog
-                 .select(CatalogToTIC_v8.catalogid)
+                 .select(CatalogToTIC_v8.catalogid,
+                         SDSS_APOGEE_AllStarMerge_r13.apogee_id,
+                         SDSS_APOGEE_AllStarMerge_r13.nvisits,
+                         SDSS_APOGEE_AllStarMerge_r13.ra.alias('allstarmerge_ra'),
+                         SDSS_APOGEE_AllStarMerge_r13.dec.alias('allstarmerge_dec'),
+                         SDSS_APOGEE_AllStarMerge_r13.pmra.alias('allstarmerge_pmra'),
+                         SDSS_APOGEE_AllStarMerge_r13.pmdec.alias('allstarmerge_pmdec'),
+                         SDSS_APOGEE_AllStarMerge_r13.h,
+                         SDSS_APOGEE_AllStarMerge_r13.baseline,
+                         SDSS_APOGEE_AllStarMerge_r13.fields)
                  .join(CatalogToTIC_v8,
                        on=(Catalog.catalogid == CatalogToTIC_v8.catalogid))
                  .join(TIC_v8,
@@ -502,6 +553,35 @@ class MWM_RV_Long_FPS_Carton(BaseCarton):
                                                        query_region[2])))
         return query
 
+    def post_process(self, model):
+        """
+        If H>10.8 then use bright_<nn>x2, otherwise use bright_<nn>x1,
+        where <nn> = 3*ceiling((18-nvisits)/3)
+        """
+
+        cursor = self.database.execute_sql(
+            "select catalogid, nvisits, h from " +
+            " sandbox.temp_mwm_rv_long_fps ;")
+
+        output = cursor.fetchall()
+
+        for i in range(len(output)):
+            current_catalogid = output[i][0]
+            current_nvisits = output[i][1]
+            current_h = output[i][2]
+
+            nn = 3 * math.ceil((18 - current_nvisits) / 3)
+            if(current_h > 10.8):
+                current_cadence = 'bright_' + str(nn) + 'x2'
+            else:
+                current_cadence = 'bright_' + str(nn) + 'x1'
+
+            if current_cadence is not None:
+                self.database.execute_sql(
+                    " update sandbox.temp_mwm_rv_long_fps " +
+                    " set cadence = '" + current_cadence + "'"
+                    " where catalogid = " + str(current_catalogid) + ";")
+
 
 # 2.2.2. Short Baseline (Fresh Targets)
 #
@@ -649,7 +729,8 @@ class MWM_RV_Short_RM_Carton(BaseCarton):
     """
     name = 'mwm_rv_short_rm'
     category = 'science'
-    cadence = None
+    instrument = 'APOGEE'
+    cadence = None  # plate only
     program = 'mwm_rv'
     mapper = 'MWM'
     priority = 2571
@@ -695,7 +776,12 @@ class MWM_RV_Short_RM_Carton(BaseCarton):
          .create_table(RadialQuery.__name__, temporary=True))
 
         query = (CatalogToTIC_v8
-                 .select(RadialQuery.c.catalogid)
+                 .select(RadialQuery.c.catalogid,
+                         Gaia_DR2.ra.alias('gaia_dr2_ra'),
+                         Gaia_DR2.dec.alias('gaia_dr2_dec'),
+                         Gaia_DR2.pmra.alias('gaia_dr2_pmra'),
+                         Gaia_DR2.pmdec.alias('gaia_dr2_pmdec'),
+                         TwoMassPSC.h_m.alias('twomass_h_m'))
                  .join(RadialQuery,
                        on=(RadialQuery.c.catalogid == CatalogToTIC_v8.catalogid))
                  .join(TIC_v8, on=(CatalogToTIC_v8.target_id == TIC_v8.id))
@@ -780,7 +866,8 @@ class MWM_RV_Short_Bplates_Carton(BaseCarton):
 """
     name = 'mwm_rv_short_bplates'
     category = 'science'
-    cadence = None
+    instrument = 'APOGEE'
+    cadence = None  # plate only
     program = 'mwm_rv'
     mapper = 'MWM'
     priority = 2576
@@ -835,7 +922,12 @@ class MWM_RV_Short_Bplates_Carton(BaseCarton):
 # We use *mwm_rv_short_condition to unpack the tuple mwm_rv_short_condition.
 # However, ra_dec_condition is not a tuple so it does not have a * in the front.
         query = (Catalog
-                 .select(CatalogToTIC_v8.catalogid)
+                 .select(CatalogToTIC_v8.catalogid,
+                         Gaia_DR2.ra.alias('gaia_dr2_ra'),
+                         Gaia_DR2.dec.alias('gaia_dr2_dec'),
+                         Gaia_DR2.pmra.alias('gaia_dr2_pmra'),
+                         Gaia_DR2.pmdec.alias('gaia_dr2_pmdec'),
+                         TwoMassPSC.h_m.alias('twomass_h_m'))
                  .join(CatalogToTIC_v8,
                        on=(Catalog.catalogid == CatalogToTIC_v8.catalogid))
                  .join(TIC_v8, on=(CatalogToTIC_v8.target_id == TIC_v8.id))
@@ -893,7 +985,8 @@ class MWM_RV_Short_FPS_Carton(BaseCarton):
     """
     name = 'mwm_rv_short_fps'
     category = 'science'
-    cadence = None
+    instrument = 'APOGEE'
+    cadence = 'bright_18x1'
     program = 'mwm_rv'
     mapper = 'MWM'
     priority = 2510
@@ -901,7 +994,12 @@ class MWM_RV_Short_FPS_Carton(BaseCarton):
     def build_query(self, version_id, query_region=None):
 
         query = (Catalog
-                 .select(CatalogToTIC_v8.catalogid)
+                 .select(CatalogToTIC_v8.catalogid,
+                         Gaia_DR2.ra.alias('gaia_dr2_ra'),
+                         Gaia_DR2.dec.alias('gaia_dr2_dec'),
+                         Gaia_DR2.pmra.alias('gaia_dr2_pmra'),
+                         Gaia_DR2.pmdec.alias('gaia_dr2_pmdec'),
+                         TwoMassPSC.h_m.alias('twomass_h_m'))
                  .join(CatalogToTIC_v8,
                        on=(Catalog.catalogid == CatalogToTIC_v8.catalogid))
                  .join(TIC_v8, on=(CatalogToTIC_v8.target_id == TIC_v8.id))
